/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.plugin.io;

import java.io.PrintWriter;
import java.util.List;

import org.odftoolkit.odfdom.OdfFileDom;
import org.odftoolkit.odfdom.doc.office.OdfOfficeAutomaticStyles;
import org.odftoolkit.odfdom.doc.office.OdfOfficeMasterStyles;
import org.odftoolkit.odfdom.doc.style.OdfStyle;
import org.odftoolkit.odfdom.doc.style.OdfStyleFooter;
import org.odftoolkit.odfdom.doc.style.OdfStyleFooterLeft;
import org.odftoolkit.odfdom.doc.style.OdfStyleHeader;
import org.odftoolkit.odfdom.doc.style.OdfStyleHeaderLeft;
import org.odftoolkit.odfdom.doc.style.OdfStyleMasterPage;
import org.odftoolkit.odfdom.doc.style.OdfStylePageLayout;
import org.odftoolkit.odfdom.doc.style.OdfStyleParagraphProperties;
import org.odftoolkit.odfdom.doc.style.OdfStyleTableCellProperties;
import org.odftoolkit.odfdom.doc.table.OdfTableCell;
import org.odftoolkit.odfdom.doc.table.OdfTableRow;
import org.odftoolkit.odfdom.dom.element.style.StylePageLayoutPropertiesElement;
import org.odftoolkit.odfdom.dom.style.OdfStyleFamily;
import org.opengion.fukurou.util.HybsEntry;
import org.opengion.hayabusa.common.HybsSystemException;
import org.opengion.hayabusa.db.DBTableModel;
import org.w3c.dom.Node;

/**
 * Calcファイルの書き出しクラスです。
 *
 * DefaultTableWriter を継承していますので，ラベル，名前，データの出力部のみ
 * オーバーライドして，OpenOfficeのCalcファイルの出力機能を実現しています。
 *
 * @og.group ファイル出力
 * 
 * @version  5.0
 * @author	 Hiroki Nakamura
 * @since    JDK6.0,
 */
public class TableWriter_CalcDef extends TableWriter_Calc {
	//* このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "4.0.0 (2005/08/31)" ;

	private static final int INIT_DATA_SIZE	= 25;
	
	private int dataSize	= INIT_DATA_SIZE;

	/**
	 * PrintWriter に DBTableModelのテーブル情報を書き込みます。
	 * このクラスでは，データを ダブルコーテーション（"）で囲みます。
	 * PrintWriter に DBTableModelのテーブル情報を書き込みます。
	 *
	 * @param	table DBTableModel
	 * @param	writer PrintWriter
	 */
	protected void writeData( final DBTableModel table,final PrintWriter writer ) {
		for( int r = 0; r < dataSize; r++ ) {
			OdfTableRow row = new OdfTableRow( contentDom );
			
			if( useNumber ) {
				String rowNo = "{@ROWNO_" + String.valueOf( r ) + "}";
				row.appendCell( createTextCell( contentDom, rowNo, false, true ) );
			}

			for( int i = 0; i < numberOfColumns; i++ ) {
				int clm = clmNo[i];
				String val = table.getColumnName( clm );
				val = "{@" + val + "_" + String.valueOf( r ) + "}";
				row.appendCell( createTextCell( contentDom, val, false, false ) );
				row.setStyleName( "ro1" );
				sheet.appendRow( row );
			}
		}
	}

	/**
	 * テキストコンテンツ用のセルを生成する
	 * 
	 * @param contentDom
	 * @param content
	 * @param isCellTypeNumber
	 * @param isNumberList
	 * @return OdfTableCell
	 */
	protected OdfTableCell createTextCell( OdfFileDom contentDom, String content, Boolean isCellTypeNumber, Boolean isNumberList ) {
		OdfTableCell cell = super.createTextCell( contentDom, content, isCellTypeNumber, isNumberList );
		if( isNumberList ) {
			OdfStyle style = contentAutoStyles.newStyle( OdfStyleFamily.TableCell );
			style.setProperty( OdfStyleTableCellProperties.TextAlignSource, "fix" );
			style.setProperty( OdfStyleTableCellProperties.RepeatContent, "false" );
			style.setProperty( OdfStyleParagraphProperties.TextAlign, "end" );
			style.setProperty( OdfStyleParagraphProperties.MarginRight, "0cm" );
			String cellStyleName = style.getStyleNameAttribute();
			cell.setStyleName( cellStyleName );
		}
		return cell;
	}

	/**
	 * デフォルトで用意されているStylesを調整します。
	 *
	 * ヘッダー表示しない
	 * フッターを数字のみにして、右端に出す
	 * ページレイアウトを横にする
	 * ページの設定を、拡大縮小モードを「印刷範囲をページ数に合わせる」に設定(1ページ)
	 */
	protected void resetAutoStylesAndMasterStyles() {
		try {
			// AutomaticStyles調整
			OdfOfficeAutomaticStyles oas = wb.getStylesDom().getAutomaticStyles();
			OdfStylePageLayout spl = oas.getPageLayout( "pm1" );
			spl.setProperty( StylePageLayoutPropertiesElement.PageHeight, "21.00cm" );
			spl.setProperty( StylePageLayoutPropertiesElement.PageWidth, "29.70cm" );
			spl.setProperty( StylePageLayoutPropertiesElement.PrintOrientation, "landscape" );
			spl.setProperty( StylePageLayoutPropertiesElement.ScaleToPages, "1" );

			// MasterStyles調整
			OdfOfficeMasterStyles oms = wb.getOfficeMasterStyles();
			OdfStyleMasterPage smp = oms.getMasterPage( "Default" );

			// MasterPageのデフォルトで用意されているノードを削除
			Node fcd = smp.getFirstChild();
			while( fcd != null ) {
				smp.removeChild( fcd );
				fcd = smp.getFirstChild();
			}

			// MasterPageのノードを定義と追加
			OdfStyleHeader sh = new OdfStyleHeader( wb.getStylesDom() );
			OdfStyleHeaderLeft shl = new OdfStyleHeaderLeft( wb.getStylesDom() );
			sh.setStyleDisplayAttribute( false );
			shl.setStyleDisplayAttribute( false );
			smp.appendChild( sh );
			smp.appendChild( shl );
			OdfStyleFooter sf = new OdfStyleFooter( wb.getStylesDom() );
			OdfStyleFooterLeft sfl = new OdfStyleFooterLeft( wb.getStylesDom() );
			sf.newStyleRegionRightElement().newTextPElement().newTextPageNumberElement();
			sfl.setStyleDisplayAttribute( false );
			smp.appendChild( sf );
			smp.appendChild( sfl );
		}
		catch( Exception ex ) {
			String errMsg = "AutomaticStylesとMasterStyles調整できません";
			throw new HybsSystemException( errMsg, ex );
		}
	}

	/**
	 * パラメーターリストをセットします。
	 * 内部は、HybsEntry クラスを持っています。
	 * 引数が、null の場合は、何もしません。
	 * 
	 * @param   listParam List<HybsEntry> パラメーターリスト
	 */
	public void setParam( final List<HybsEntry> listParam ) {
		if( listParam != null && !listParam.isEmpty() ) {
			for( HybsEntry entry : listParam ) {
				if( "Size".equalsIgnoreCase( entry.getKey() ) ) {
					dataSize = Integer.parseInt( entry.getValue() );
				}
			}
		}
	}
}
